"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const ts = __importStar(require("@terascope/utils"));
const connector_utils_1 = require("../connector-utils");
const utils_1 = require("./utils");
const prom_metrics_api_1 = require("./prom-metrics/prom-metrics-api");
/*
 * This module controls the API endpoints that are exposed under context.apis.
 */
function registerApis(context) {
    const foundationConfig = context.sysconfig.terafoundation;
    const events = new events_1.EventEmitter();
    context.logger = (0, utils_1.createRootLogger)(context);
    // connection cache
    const connections = Object.create(null);
    const foundationApis = {
        async createClient(options) {
            const { type, endpoint = 'default', cached } = options;
            // If it's acceptable to use a cached connection just return instead
            // of creating a new one
            const key = `${type}:${endpoint}`;
            // Location in the configuration where we look for connectors.
            const { connectors } = foundationConfig;
            if (cached && Object.prototype.hasOwnProperty.call(connections, key)) {
                return connections[key];
            }
            if (Object.prototype.hasOwnProperty.call(connectors, type)) {
                context.logger.info(`creating connection for ${type}`);
                let moduleConfig = {};
                if (Object.prototype.hasOwnProperty.call(connectors[type], endpoint)) {
                    moduleConfig = Object.assign({}, foundationConfig.connectors[type][endpoint]);
                    // If an endpoint was specified and doesn't exist we need to error.
                }
                else if (endpoint) {
                    throw new Error(`No ${type} endpoint configuration found for ${endpoint}`);
                }
                const connection = await (0, connector_utils_1.createClient)(type, moduleConfig, context.logger, options);
                if (cached) {
                    connections[key] = connection;
                }
                return connection;
            }
            throw new Error(`No connection configuration found for ${type}`);
        },
        getConnection(options) {
            const { type, endpoint = 'default', cached } = options;
            // If it's acceptable to use a cached connection just return instead
            // of creating a new one
            const key = `${type}:${endpoint}`;
            // Location in the configuration where we look for connectors.
            const { connectors } = foundationConfig;
            if (cached && Object.prototype.hasOwnProperty.call(connections, key)) {
                return connections[key];
            }
            if (Object.prototype.hasOwnProperty.call(connectors, type)) {
                context.logger.info(`creating connection for ${type}`);
                let moduleConfig = {};
                if (Object.prototype.hasOwnProperty.call(connectors[type], endpoint)) {
                    moduleConfig = Object.assign({}, foundationConfig.connectors[type][endpoint]);
                    // If an endpoint was specified and doesn't exist we need to error.
                }
                else if (endpoint) {
                    throw new Error(`No ${type} endpoint configuration found for ${endpoint}`);
                }
                const connection = (0, connector_utils_1.createConnection)(type, moduleConfig, context.logger, options);
                if (cached) {
                    connections[key] = connection;
                }
                return connection;
            }
            throw new Error(`No connection configuration found for ${type}`);
        },
        makeLogger(...args) {
            // If there is already a logger defined we're just creating a
            // child logger using the same config.
            const childLogger = context.logger.child(ts.isPlainObject(args[0])
                ? args[0]
                : args[2]);
            // add flush fn to the new logger
            childLogger.flush = context.logger.flush;
            return childLogger;
        },
        getSystemEvents() {
            return events;
        },
        startWorkers(num, envOptions) {
            const { cluster } = context;
            // default assignment is set to worker
            // service_context acts as a dictionary to know what env variables
            // are needed on restarts and crashes
            const env = {
                assignment: 'worker',
                service_context: JSON.stringify({ assignment: 'worker' })
            };
            if (envOptions) {
                Object.assign(env, envOptions);
                env.service_context = JSON.stringify(envOptions);
            }
            const workers = [];
            if (cluster.isMaster) {
                context.logger.info(`Starting ${num} ${env.assignment}`);
                ts.times(num, () => {
                    const worker = cluster.fork(env);
                    // for cluster master reference, when a worker dies, you
                    // don't have access to its env at master level
                    Object.assign(worker, env);
                    workers.push(worker);
                });
            }
            return workers;
        },
        promMetrics: new prom_metrics_api_1.PromMetrics(context, context.logger)
    };
    function _registerFoundationAPIs() {
        registerAPI('foundation', foundationApis);
    }
    // Accessing these APIs directly under context.foundation is deprecated.
    function _registerLegacyAPIs() {
        const { getSystemEvents, ...legacyApis } = foundationApis;
        context.foundation = {
            ...legacyApis,
            getEventEmitter: getSystemEvents,
        };
    }
    /*
     * Used by modules to register API endpoints that can be used elsewhere
     * in the system.
     * @param {string} name - The module name being registered
     * @param {string} api - Object containing the functions being exposed as the API
     */
    function registerAPI(name, api) {
        if (Object.prototype.hasOwnProperty.call(context.apis, name)) {
            throw new Error(`Registration of API endpoints for module ${name} can only occur once`);
        }
        else {
            context.apis[name] = api;
        }
    }
    // This exposes the registerAPI function to the rest of the system.
    context.apis = {
        registerAPI
    };
    _registerFoundationAPIs();
    _registerLegacyAPIs();
    /*
     * Setup proxy for 'foundation.promMetrics' here to have access to 'PromMetrics' functions.
     * This proxy allows the interception of the 'promMetrics' function calls in the case that
     * 'PromMetrics' has not been initialized.
    */
    const promMetricsProxy = new Proxy(context.apis.foundation.promMetrics, {
        get(promMetrics, funcName) {
            const apiExists = promMetrics.verifyAPI();
            if (apiExists) {
                if (funcName === 'init') {
                    throw new Error('Prom metrics API cannot be initialized more than once.');
                }
                return promMetrics[funcName];
            }
            if (funcName === 'init') {
                return promMetrics[funcName];
            }
            if (funcName === 'hasMetric' || funcName === 'deleteMetric') {
                return () => false;
            }
            if (funcName === 'set' || funcName === 'addMetric'
                || funcName === 'addSummary' || funcName === 'inc'
                || funcName === 'dec' || funcName === 'observe'
                || funcName === 'shutdown') {
                return () => {
                    /// return empty function
                };
            }
            return promMetrics[funcName];
        }
    });
    /// Set the global promMetrics to the promMetricsProxy to override functions everywhere
    context.apis.foundation.promMetrics = promMetricsProxy;
}
exports.default = registerApis;
//# sourceMappingURL=index.js.map