"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prom_client_1 = __importDefault(require("prom-client"));
const express_1 = __importDefault(require("express"));
class Exporter {
    constructor(logger) {
        this.logger = logger.child({ module: 'prometheus_exporter' });
    }
    async create(promMetricsAPIConfig) {
        try {
            if (promMetricsAPIConfig.default_metrics) {
                const defaultMetricsConfig = {
                    prefix: promMetricsAPIConfig.prefix,
                    labels: promMetricsAPIConfig.labels
                };
                prom_client_1.default.collectDefaultMetrics(defaultMetricsConfig);
            }
            const { port } = promMetricsAPIConfig;
            const app = (0, express_1.default)();
            app.get('/', async (_req, res) => {
                res.status(404).send('See the \'/metrics\' endpoint for the teraslice metric exporter\n');
            });
            app.get('/metrics', async (_req, res) => {
                res.set('Content-Type', prom_client_1.default.register.contentType);
                res.end(await prom_client_1.default.register.metrics());
            });
            this.metricServer = app.listen(port);
        }
        catch (err) {
            this.logger.error('Prometheus exporter creation failed: ', err);
        }
    }
    async shutdown() {
        this.metricServer.close();
    }
    async deleteMetric(name) {
        prom_client_1.default.register.removeSingleMetric(name);
    }
}
exports.default = Exporter;
//# sourceMappingURL=exporter.js.map